@@ -17,11 +17,10 @@ from anndata import AnnData
 from matplotlib.cm import ScalarMappable
 from matplotlib.colors import ListedColormap, Normalize
 from scanpy._settings import settings as sc_settings
-from spatialdata import get_extent
-from spatialdata.models import PointsModel, get_table_keys
-from spatialdata.transformations import (
-    set_transformation,
-)
+from spatialdata import get_extent, get_values, join_spatialelement_table
+from spatialdata.models import PointsModel, ShapesModel, get_table_keys
+from spatialdata.transformations import get_transformation, set_transformation
+from spatialdata.transformations.transformations import Identity
 from xarray import DataTree
 
 from spatialdata_plot._logging import logger
@@ -44,6 +43,7 @@ from spatialdata_plot.pl.utils import (
     _get_colors_for_categorical_obs,
     _get_extent_and_range_for_datashader_canvas,
     _get_linear_colormap,
+    _get_transformation_matrix_for_datashader,
     _is_coercable_to_float,
     _map_color_seg,
     _maybe_set_colors,
@@ -70,19 +70,25 @@ def _render_shapes(
     element = render_params.element
     col_for_color = render_params.col_for_color
     groups = render_params.groups
+    table_layer = render_params.table_layer
 
     sdata_filt = sdata.filter_by_coordinate_system(
         coordinate_system=coordinate_system,
         filter_tables=bool(render_params.table_name),
     )
 
-    shapes = sdata[element]
-
     if (table_name := render_params.table_name) is None:
         table = None
+        shapes = sdata_filt[element]
     else:
-        _, region_key, _ = get_table_keys(sdata[table_name])
-        table = sdata[table_name][sdata[table_name].obs[region_key].isin([element])]
+        element_dict, joined_table = join_spatialelement_table(
+            sdata, spatial_element_names=element, table_name=table_name, how="inner"
+        )
+        sdata_filt[element] = shapes = element_dict[element]
+        joined_table.uns["spatialdata_attrs"]["region"] = (
+            joined_table.obs[joined_table.uns["spatialdata_attrs"]["region_key"]].unique().tolist()
+        )
+        sdata_filt[table_name] = table = joined_table
 
     if (
         col_for_color is not None
@@ -110,6 +116,7 @@ def _render_shapes(
         na_color=render_params.color or render_params.cmap_params.na_color,
         cmap_params=render_params.cmap_params,
         table_name=table_name,
+        table_layer=table_layer,
     )
 
     values_are_categorical = color_source_vector is not None
@@ -148,7 +155,7 @@ def _render_shapes(
         colorbar = False if col_for_color is None else legend_params.colorbar
 
     # Apply the transformation to the PatchCollection's paths
-    trans, _ = _prepare_transformation(sdata_filt.shapes[element], coordinate_system)
+    trans, trans_data = _prepare_transformation(sdata_filt.shapes[element], coordinate_system)
 
     shapes = gpd.GeoDataFrame(shapes, geometry="geometry")
 
@@ -168,14 +175,6 @@ def _render_shapes(
         )
 
     if method == "datashader":
-        trans += ax.transData
-
-        plot_width, plot_height, x_ext, y_ext, factor = _get_extent_and_range_for_datashader_canvas(
-            sdata_filt.shapes[element], coordinate_system, ax, fig_params
-        )
-
-        cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_ext, y_range=y_ext)
-
         _geometry = shapes["geometry"]
         is_point = _geometry.type == "Point"
 
@@ -184,19 +183,31 @@ def _render_shapes(
             scale = shapes[is_point]["radius"] * render_params.scale
             sdata_filt.shapes[element].loc[is_point, "geometry"] = _geometry[is_point].buffer(scale.to_numpy())
 
+        # apply transformations to the individual points
+        element_trans = get_transformation(sdata_filt.shapes[element])
+        tm = _get_transformation_matrix_for_datashader(element_trans)
+        transformed_element = sdata_filt.shapes[element].transform(
+            lambda x: (np.hstack([x, np.ones((x.shape[0], 1))]) @ tm)[:, :2]
+        )
+        transformed_element = ShapesModel.parse(
+            gpd.GeoDataFrame(data=sdata_filt.shapes[element].drop("geometry", axis=1), geometry=transformed_element)
+        )
+
+        plot_width, plot_height, x_ext, y_ext, factor = _get_extent_and_range_for_datashader_canvas(
+            transformed_element, coordinate_system, ax, fig_params
+        )
+
+        cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_ext, y_range=y_ext)
+
         # in case we are coloring by a column in table
-        if col_for_color is not None and col_for_color not in sdata_filt.shapes[element].columns:
-            sdata_filt.shapes[element][col_for_color] = (
-                color_vector if color_source_vector is None else color_source_vector
-            )
+        if col_for_color is not None and col_for_color not in transformed_element.columns:
+            transformed_element[col_for_color] = color_vector if color_source_vector is None else color_source_vector
         # Render shapes with datashader
         color_by_categorical = col_for_color is not None and color_source_vector is not None
         aggregate_with_reduction = None
         if col_for_color is not None and (render_params.groups is None or len(render_params.groups) > 1):
             if color_by_categorical:
-                agg = cvs.polygons(
-                    sdata_filt.shapes[element], geometry="geometry", agg=ds.by(col_for_color, ds.count())
-                )
+                agg = cvs.polygons(transformed_element, geometry="geometry", agg=ds.by(col_for_color, ds.count()))
             else:
                 reduction_name = render_params.ds_reduction if render_params.ds_reduction is not None else "mean"
                 logger.info(
@@ -204,16 +215,16 @@ def _render_shapes(
                     "to the matplotlib result."
                 )
                 agg = _datashader_aggregate_with_function(
-                    render_params.ds_reduction, cvs, sdata_filt.shapes[element], col_for_color, "shapes"
+                    render_params.ds_reduction, cvs, transformed_element, col_for_color, "shapes"
                 )
                 # save min and max values for drawing the colorbar
                 aggregate_with_reduction = (agg.min(), agg.max())
         else:
-            agg = cvs.polygons(sdata_filt.shapes[element], geometry="geometry", agg=ds.count())
+            agg = cvs.polygons(transformed_element, geometry="geometry", agg=ds.count())
         # render outlines if needed
         if (render_outlines := render_params.outline_alpha) > 0:
             agg_outlines = cvs.line(
-                sdata_filt.shapes[element],
+                transformed_element,
                 geometry="geometry",
                 line_width=render_params.outline_params.linewidth,
             )
@@ -287,13 +298,23 @@ def _render_shapes(
 
         rgba_image, trans_data = _create_image_from_datashader_result(ds_result, factor, ax)
         _cax = _ax_show_and_transform(
-            rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.fill_alpha
+            rgba_image,
+            trans_data,
+            ax,
+            zorder=render_params.zorder,
+            alpha=render_params.fill_alpha,
+            extent=x_ext + y_ext,
         )
         # render outline image if needed
         if render_outlines:
             rgba_image, trans_data = _create_image_from_datashader_result(ds_outlines, factor, ax)
             _ax_show_and_transform(
-                rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.outline_alpha
+                rgba_image,
+                trans_data,
+                ax,
+                zorder=render_params.zorder,
+                alpha=render_params.outline_alpha,
+                extent=x_ext + y_ext,
             )
 
         cax = None
@@ -330,7 +351,7 @@ def _render_shapes(
 
     if not values_are_categorical:
         # If the user passed a Normalize object with vmin/vmax we'll use those,
-        # # if not we'll use the min/max of the color_vector
+        # if not we'll use the min/max of the color_vector
         _cax.set_clim(
             vmin=render_params.cmap_params.norm.vmin or min(color_vector),
             vmax=render_params.cmap_params.norm.vmax or max(color_vector),
@@ -378,6 +399,7 @@ def _render_points(
     element = render_params.element
     col_for_color = render_params.col_for_color
     table_name = render_params.table_name
+    table_layer = render_params.table_layer
     color = render_params.color
     groups = render_params.groups
     palette = render_params.palette
@@ -390,10 +412,22 @@ def _render_points(
     points = sdata.points[element]
     coords = ["x", "y"]
 
-    if col_for_color is None or (table_name is not None and col_for_color in sdata_filt[table_name].obs.columns):
+    if table_name is not None and col_for_color not in points.columns:
+        warnings.warn(
+            f"Annotating points with {col_for_color} which is stored in the table `{table_name}`. "
+            f"To improve performance, it is advisable to store point annotations directly in the .parquet file.",
+            UserWarning,
+            stacklevel=2,
+        )
+
+    if col_for_color is None or (
+        table_name is not None
+        and (col_for_color in sdata_filt[table_name].obs.columns or col_for_color in sdata_filt[table_name].var_names)
+    ):
         points = points[coords].compute()
         if (
             col_for_color
+            and col_for_color in sdata_filt[table_name].obs.columns
             and (color_col := sdata_filt[table_name].obs[col_for_color]).dtype == "O"
             and not _is_coercable_to_float(color_col)
         ):
@@ -409,7 +443,20 @@ def _render_points(
         points = points[coords].compute()
 
     if groups is not None and col_for_color is not None:
-        points = points[points[col_for_color].isin(groups)]
+        if col_for_color in points.columns:
+            points_color_values = points[col_for_color]
+        else:
+            points_color_values = get_values(
+                value_key=col_for_color,
+                sdata=sdata_filt,
+                element_name=element,
+                table_name=table_name,
+                table_layer=table_layer,
+            )
+            points_color_values = points.merge(points_color_values, how="left", left_index=True, right_index=True)[
+                col_for_color
+            ]
+        points = points[points_color_values.isin(groups)]
         if len(points) <= 0:
             raise ValueError(f"None of the groups {groups} could be found in the column '{col_for_color}'.")
 
@@ -419,9 +466,18 @@ def _render_points(
             X=points[["x", "y"]].values, obs=points[coords].reset_index(), dtype=points[["x", "y"]].values.dtype
         )
     else:
+        adata_obs = sdata_filt[table_name].obs
+        # if the points are colored by values in X (or a different layer), add the values to obs
+        if col_for_color in sdata_filt[table_name].var_names:
+            if table_layer is None:
+                adata_obs[col_for_color] = sdata_filt[table_name][:, col_for_color].X.flatten().copy()
+            else:
+                adata_obs[col_for_color] = sdata_filt[table_name][:, col_for_color].layers[table_layer].flatten().copy()
+        if groups is not None:
+            adata_obs = adata_obs[adata_obs[col_for_color].isin(groups)]
         adata = AnnData(
             X=points[["x", "y"]].values,
-            obs=sdata_filt[table_name].obs,
+            obs=adata_obs,
             dtype=points[["x", "y"]].values.dtype,
             uns=sdata_filt[table_name].uns,
         )
@@ -439,7 +495,8 @@ def _render_points(
     )
 
     if col_for_color is not None:
-        cols = sc.get.obs_df(adata, col_for_color)
+        assert isinstance(col_for_color, str)
+        cols = sc.get.obs_df(adata, [col_for_color])
         # maybe set color based on type
         if isinstance(cols[col_for_color].dtype, pd.CategoricalDtype):
             _maybe_set_colors(
@@ -468,7 +525,7 @@ def _render_points(
     if color_source_vector is None and render_params.transfunc is not None:
         color_vector = render_params.transfunc(color_vector)
 
-    _, trans_data = _prepare_transformation(sdata.points[element], coordinate_system, ax)
+    trans, trans_data = _prepare_transformation(sdata.points[element], coordinate_system, ax)
 
     norm = copy(render_params.cmap_params.norm)
 
@@ -491,18 +548,30 @@ def _render_points(
         # use dpi/100 as a factor for cases where dpi!=100
         px = int(np.round(np.sqrt(render_params.size) * (fig_params.fig.dpi / 100)))
 
+        # apply transformations
+        transformed_element = PointsModel.parse(
+            trans.transform(sdata_filt.points[element][["x", "y"]]),
+            annotation=sdata_filt.points[element][sdata_filt.points[element].columns.drop(["x", "y"])],
+            transformations={coordinate_system: Identity()},
+        )
+
         plot_width, plot_height, x_ext, y_ext, factor = _get_extent_and_range_for_datashader_canvas(
-            sdata_filt.points[element], coordinate_system, ax, fig_params
+            transformed_element, coordinate_system, ax, fig_params
         )
 
         # use datashader for the visualization of points
         cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_ext, y_range=y_ext)
 
-        color_by_categorical = col_for_color is not None and points[col_for_color].values.dtype == object
+        color_by_categorical = col_for_color is not None and transformed_element[col_for_color].values.dtype in (
+            object,
+            "categorical",
+        )
+        if color_by_categorical and transformed_element[col_for_color].values.dtype == object:
+            transformed_element[col_for_color] = transformed_element[col_for_color].astype("category")
         aggregate_with_reduction = None
         if col_for_color is not None and (render_params.groups is None or len(render_params.groups) > 1):
             if color_by_categorical:
-                agg = cvs.points(sdata_filt.points[element], "x", "y", agg=ds.by(col_for_color, ds.count()))
+                agg = cvs.points(transformed_element, "x", "y", agg=ds.by(col_for_color, ds.count()))
             else:
                 reduction_name = render_params.ds_reduction if render_params.ds_reduction is not None else "sum"
                 logger.info(
@@ -510,12 +579,12 @@ def _render_points(
                     "to the matplotlib result."
                 )
                 agg = _datashader_aggregate_with_function(
-                    render_params.ds_reduction, cvs, sdata_filt.points[element], col_for_color, "points"
+                    render_params.ds_reduction, cvs, transformed_element, col_for_color, "points"
                 )
                 # save min and max values for drawing the colorbar
                 aggregate_with_reduction = (agg.min(), agg.max())
         else:
-            agg = cvs.points(sdata_filt.points[element], "x", "y", agg=ds.count())
+            agg = cvs.points(transformed_element, "x", "y", agg=ds.count())
 
         if norm.vmin is not None or norm.vmax is not None:
             norm.vmin = np.min(agg) if norm.vmin is None else norm.vmin
@@ -573,7 +642,14 @@ def _render_points(
             )
 
         rgba_image, trans_data = _create_image_from_datashader_result(ds_result, factor, ax)
-        _ax_show_and_transform(rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.alpha)
+        _ax_show_and_transform(
+            rgba_image,
+            trans_data,
+            ax,
+            zorder=render_params.zorder,
+            alpha=render_params.alpha,
+            extent=x_ext + y_ext,
+        )
 
         cax = None
         if aggregate_with_reduction is not None:
@@ -678,8 +754,10 @@ def _render_images(
             extent=extent,
         )
 
-    channels = img.coords["c"].values if render_params.channel is None else render_params.channel
+    channels = img.coords["c"].values.tolist() if render_params.channel is None else render_params.channel
 
+    # the channel parameter has been previously validated, so when not None, render_params.channel is a list
+    assert isinstance(channels, list)
     n_channels = len(channels)
 
     # True if user gave n cmaps for n channels
@@ -806,6 +884,7 @@ def _render_labels(
 ) -> None:
     element = render_params.element
     table_name = render_params.table_name
+    table_layer = render_params.table_layer
     palette = render_params.palette
     color = render_params.color
     groups = render_params.groups
@@ -841,7 +920,7 @@ def _render_labels(
             extent=extent,
         )
 
-        # the avove adds a useless c dimension of 1 (y, x) -> (1, y, x)
+        # the above adds a useless c dimension of 1 (y, x) -> (1, y, x)
         label = label.squeeze()
 
     if table_name is None:
@@ -866,6 +945,7 @@ def _render_labels(
         na_color=render_params.cmap_params.na_color,
         cmap_params=render_params.cmap_params,
         table_name=table_name,
+        table_layer=table_layer,
     )
 
     def _draw_labels(seg_erosionpx: int | None, seg_boundaries: bool, alpha: float) -> matplotlib.image.AxesImage:
