@@ -1,3 +1,5 @@
+import math
+
 import dask.dataframe
 import matplotlib
 import matplotlib.pyplot as plt
@@ -7,6 +9,8 @@ import scanpy as sc
 from anndata import AnnData
 from spatialdata import SpatialData, deepcopy
 from spatialdata.models import PointsModel, TableModel
+from spatialdata.transformations import Affine, Identity, MapAxis, Scale, Sequence, Translation
+from spatialdata.transformations._utils import _set_transformations
 
 import spatialdata_plot  # noqa: F401
 from tests.conftest import DPI, PlotTester, PlotTesterMeta
@@ -107,6 +111,18 @@ class TestPoints(PlotTester, metaclass=PlotTesterMeta):
             element="blobs_points", size=40, color="instance_id", alpha=0.6, method="datashader"
         ).pl.show()
 
+    def test_plot_points_categorical_color_column_matplotlib(self, sdata_blobs: SpatialData):
+        sdata_blobs.pl.render_points("blobs_points", color="genes", method="matplotlib").pl.show()
+
+    def test_plot_points_categorical_color_column_datashader(self, sdata_blobs: SpatialData):
+        sdata_blobs.pl.render_points("blobs_points", color="genes", method="datashader").pl.show()
+
+    def test_plot_points_continuous_color_column_matplotlib(self, sdata_blobs: SpatialData):
+        sdata_blobs.pl.render_points("blobs_points", color="instance_id", method="matplotlib").pl.show()
+
+    def test_plot_points_continuous_color_column_datashader(self, sdata_blobs: SpatialData):
+        sdata_blobs.pl.render_points("blobs_points", color="instance_id", method="datashader").pl.show()
+
     def test_plot_datashader_matplotlib_stack(self, sdata_blobs: SpatialData):
         sdata_blobs.pl.render_points(
             element="blobs_points", size=40, color="red", method="datashader"
@@ -173,3 +189,113 @@ class TestPoints(PlotTester, metaclass=PlotTesterMeta):
         sdata_blobs.pl.render_points(element="blobs_points", size=400, color="blue").pl.render_points(
             element="blobs_points", size=400, color="yellow", method="datashader", alpha=0.8
         ).pl.show(dpi=200)
+
+    def test_plot_points_transformed_ds_agrees_with_mpl(self):
+        sdata = SpatialData(
+            points={
+                "points1": PointsModel.parse(
+                    pd.DataFrame({"y": [0, 0, 10, 10, 4, 6, 4, 6], "x": [0, 10, 10, 0, 4, 6, 6, 4]}),
+                    transformations={"global": Scale([2, 2], ("y", "x"))},
+                )
+            },
+        )
+        sdata.pl.render_points("points1", method="matplotlib", size=50, color="lightgrey").pl.render_points(
+            "points1", method="datashader", size=10, color="red"
+        ).pl.show()
+
+    def test_plot_datashader_can_transform_points(self, sdata_blobs: SpatialData):
+        theta = math.pi / 1.7
+        rotation = Affine(
+            [
+                [math.cos(theta), -math.sin(theta), 0],
+                [math.sin(theta), math.cos(theta), 0],
+                [0, 0, 1],
+            ],
+            input_axes=("x", "y"),
+            output_axes=("x", "y"),
+        )
+
+        scale = Scale([-1.3, 1.8], axes=("x", "y"))
+        identity = Identity()
+        mapaxis = MapAxis({"x": "y", "y": "x"})
+        translation = Translation([20, -65], ("x", "y"))
+        seq = Sequence([mapaxis, scale, identity, translation, rotation])
+
+        _set_transformations(sdata_blobs["blobs_points"], {"global": seq})
+
+        sdata_blobs.pl.render_points("blobs_points", method="datashader", color="black", size=5).pl.show()
+
+    def test_plot_can_annotate_points_with_table_obs(self, sdata_blobs: SpatialData):
+        nrows, ncols = 200, 3
+        feature_matrix = RNG.random((nrows, ncols))
+        var_names = [f"feature{i}" for i in range(ncols)]
+
+        obs_indices = sdata_blobs["blobs_points"].index
+
+        obs = pd.DataFrame()
+        obs["instance_id"] = obs_indices
+        obs["region"] = "blobs_points"
+        obs["region"].astype("category")
+        obs["extra_feature"] = [1, 2] * 100
+
+        table = AnnData(X=feature_matrix, var=pd.DataFrame(index=var_names), obs=obs)
+        table = TableModel.parse(table, region="blobs_points", region_key="region", instance_key="instance_id")
+        sdata_blobs["points_table"] = table
+
+        sdata_blobs.pl.render_points("blobs_points", color="extra_feature", size=10).pl.show()
+
+    def test_plot_can_annotate_points_with_table_X(self, sdata_blobs: SpatialData):
+        nrows, ncols = 200, 3
+        feature_matrix = RNG.random((nrows, ncols))
+        var_names = [f"feature{i}" for i in range(ncols)]
+
+        obs_indices = sdata_blobs["blobs_points"].index
+
+        obs = pd.DataFrame()
+        obs["instance_id"] = obs_indices
+        obs["region"] = "blobs_points"
+        obs["region"].astype("category")
+
+        table = AnnData(X=feature_matrix, var=pd.DataFrame(index=var_names), obs=obs)
+        table = TableModel.parse(table, region="blobs_points", region_key="region", instance_key="instance_id")
+        sdata_blobs["points_table"] = table
+
+        sdata_blobs.pl.render_points("blobs_points", color="feature0", size=10).pl.show()
+
+    def test_plot_can_annotate_points_with_table_and_groups(self, sdata_blobs: SpatialData):
+        nrows, ncols = 200, 3
+        feature_matrix = RNG.random((nrows, ncols))
+        var_names = [f"feature{i}" for i in range(ncols)]
+
+        obs_indices = sdata_blobs["blobs_points"].index
+
+        obs = pd.DataFrame()
+        obs["instance_id"] = obs_indices
+        obs["region"] = "blobs_points"
+        obs["region"].astype("category")
+        obs["extra_feature_cat"] = ["one", "two"] * 100
+
+        table = AnnData(X=feature_matrix, var=pd.DataFrame(index=var_names), obs=obs)
+        table = TableModel.parse(table, region="blobs_points", region_key="region", instance_key="instance_id")
+        sdata_blobs["points_table"] = table
+
+        sdata_blobs.pl.render_points("blobs_points", color="extra_feature_cat", groups="two", size=10).pl.show()
+
+    def test_plot_can_annotate_points_with_table_layer(self, sdata_blobs: SpatialData):
+        nrows, ncols = 200, 3
+        feature_matrix = RNG.random((nrows, ncols))
+        var_names = [f"feature{i}" for i in range(ncols)]
+
+        obs_indices = sdata_blobs["blobs_points"].index
+
+        obs = pd.DataFrame()
+        obs["instance_id"] = obs_indices
+        obs["region"] = "blobs_points"
+        obs["region"].astype("category")
+
+        table = AnnData(X=feature_matrix, var=pd.DataFrame(index=var_names), obs=obs)
+        table = TableModel.parse(table, region="blobs_points", region_key="region", instance_key="instance_id")
+        sdata_blobs["points_table"] = table
+        sdata_blobs["points_table"].layers["normalized"] = RNG.random((nrows, ncols))
+
+        sdata_blobs.pl.render_points("blobs_points", color="feature0", size=10, table_layer="normalized").pl.show()
