@@ -1,3 +1,5 @@
+import math
+
 import anndata
 import geopandas as gpd
 import matplotlib
@@ -10,6 +12,8 @@ from matplotlib.colors import Normalize
 from shapely.geometry import MultiPolygon, Point, Polygon
 from spatialdata import SpatialData, deepcopy
 from spatialdata.models import ShapesModel, TableModel
+from spatialdata.transformations import Affine, Identity, MapAxis, Scale, Sequence, Translation
+from spatialdata.transformations._utils import _set_transformations
 
 import spatialdata_plot  # noqa: F401
 from tests.conftest import DPI, PlotTester, PlotTesterMeta
@@ -165,7 +169,6 @@ class TestShapes(PlotTester, metaclass=PlotTesterMeta):
         cropped_blob.pl.render_shapes().pl.show()
 
     def test_plot_can_plot_with_annotation_despite_random_shuffling(self, sdata_blobs: SpatialData):
-        new_table = sdata_blobs["table"].copy()
         sdata_blobs["table"].obs["region"] = "blobs_circles"
         new_table = sdata_blobs["table"][:5]
         new_table.uns["spatialdata_attrs"]["region"] = "blobs_circles"
@@ -377,3 +380,97 @@ class TestShapes(PlotTester, metaclass=PlotTesterMeta):
         sdata_blobs.pl.render_shapes(
             "blobs_circles", color="dummy_gene_expression", norm=norm, table_name="new_table"
         ).pl.show()
+
+    def test_plot_datashader_can_transform_polygons(self, sdata_blobs: SpatialData):
+        theta = math.pi / 1.7
+        rotation = Affine(
+            [
+                [math.cos(theta), -math.sin(theta), 0],
+                [math.sin(theta), math.cos(theta), 0],
+                [0, 0, 1],
+            ],
+            input_axes=("x", "y"),
+            output_axes=("x", "y"),
+        )
+
+        scale = Scale([-1.3, 1.8], axes=("x", "y"))
+        identity = Identity()
+        mapaxis = MapAxis({"x": "y", "y": "x"})
+        translation = Translation([20, -65], ("x", "y"))
+        seq = Sequence([mapaxis, scale, identity, translation, rotation])
+
+        _set_transformations(sdata_blobs["blobs_polygons"], {"global": seq})
+
+        sdata_blobs.pl.render_shapes("blobs_polygons", method="datashader", outline_alpha=1.0).pl.show()
+
+    def test_plot_datashader_can_transform_multipolygons(self, sdata_blobs: SpatialData):
+        theta = math.pi / 1.7
+        rotation = Affine(
+            [
+                [math.cos(theta), -math.sin(theta), 0],
+                [math.sin(theta), math.cos(theta), 0],
+                [0, 0, 1],
+            ],
+            input_axes=("x", "y"),
+            output_axes=("x", "y"),
+        )
+
+        scale = Scale([-1.3, 1.8], axes=("x", "y"))
+        identity = Identity()
+        mapaxis = MapAxis({"x": "y", "y": "x"})
+        translation = Translation([20, -65], ("x", "y"))
+        seq = Sequence([mapaxis, scale, identity, translation, rotation])
+
+        _set_transformations(sdata_blobs["blobs_multipolygons"], {"global": seq})
+
+        sdata_blobs.pl.render_shapes("blobs_multipolygons", method="datashader", outline_alpha=1.0).pl.show()
+
+    def test_plot_datashader_can_transform_circles(self, sdata_blobs: SpatialData):
+        theta = math.pi / 1.7
+        rotation = Affine(
+            [
+                [math.cos(theta), -math.sin(theta), 0],
+                [math.sin(theta), math.cos(theta), 0],
+                [0, 0, 1],
+            ],
+            input_axes=("x", "y"),
+            output_axes=("x", "y"),
+        )
+
+        scale = Scale([-1.3, 1.8], axes=("x", "y"))
+        identity = Identity()
+        mapaxis = MapAxis({"x": "y", "y": "x"})
+        translation = Translation([20, -65], ("x", "y"))
+        seq = Sequence([mapaxis, scale, identity, translation, rotation])
+
+        _set_transformations(sdata_blobs["blobs_circles"], {"global": seq})
+
+        sdata_blobs.pl.render_shapes("blobs_circles", method="datashader", outline_alpha=1.0).pl.show()
+
+    def test_plot_can_do_non_matching_table(self, sdata_blobs: SpatialData):
+        table_shapes = sdata_blobs["table"][:3].copy()
+        table_shapes.obs.instance_id = list(range(3))
+        table_shapes.obs["region"] = "blobs_circles"
+        table_shapes.uns["spatialdata_attrs"]["region"] = "blobs_circles"
+        sdata_blobs["new_table"] = table_shapes
+
+        sdata_blobs.pl.render_shapes("blobs_circles", color="instance_id").pl.show()
+
+    def test_plot_can_annotate_shapes_with_table_layer(self, sdata_blobs: SpatialData):
+        nrows, ncols = 5, 3
+        feature_matrix = RNG.random((nrows, ncols))
+        var_names = [f"feature{i}" for i in range(ncols)]
+
+        obs_indices = sdata_blobs["blobs_circles"].index
+
+        obs = pd.DataFrame()
+        obs["instance_id"] = obs_indices
+        obs["region"] = "blobs_circles"
+        obs["region"].astype("category")
+
+        table = AnnData(X=feature_matrix, var=pd.DataFrame(index=var_names), obs=obs)
+        table = TableModel.parse(table, region="blobs_circles", region_key="region", instance_key="instance_id")
+        sdata_blobs["circle_table"] = table
+        sdata_blobs["circle_table"].layers["normalized"] = RNG.random((nrows, ncols))
+
+        sdata_blobs.pl.render_shapes("blobs_circles", color="feature0", table_layer="normalized").pl.show()
